/******************************************************************
 ** image_math.h
 ** Namespace: image_math
 ******************************************************************/


#if !defined(IMAGE_MATH_H)
#define IMAGE_MATH_H 1

#include "point.h"
#include "image.h"

namespace stdi {

  namespace image_math {
    
    /**
       \return sqrt(a*a+b*b)
    */
    float norm(float a, float b);
    /**
       \return a*a+b*b
    */
    float norm2(float a, float b);
    
    /**
       \param a dx
       \param b dy
       \return angle in degree, formed by dx and dy
    */
    float angle(float a, float b);
    
    /**
       \param a dx
       \param b dy
       \return index of direction of the angle formed by dx and dy, 0<=index<=7, anti-clockwise;
    */
    byte direction(float a, float b);
    /**
       \param a angle in degree
       \return index of direction of the angle, 0<=index<=7, anti-clockwise; E.g. when -22.5<=a<=22.5, index(a) = 0;
    */
    byte direction(float a);
    
    
    PointArray findZeros(const Image &I);
    PointArray findZeros(const Image &Ix, const Image &Iy);

    /**
       \param I input image
       \param p percentage value, should be within [0,1]
       \return the intensity threshold for the top p*100 percent pixels in the input image
    */
    float topByPercentage(const Image & I, float p);  
    
    /**
       \param I input image
       \param n integer number, should be within [0,num_pixel]
       \return the intensity threshold for the top n pixels in the input image
    */
    float topByPixelNumber(const Image & I, int n);
    
    /**
       \param I input image
       \param p percentage value, should be within [0,1]
       \return the mean intensity of top p*100 percent pixels in the input image
    */
    float meanOfTopByPercentage(const Image & I, float p);
    
    /**
       \param va input array
       \param p percentage, should be within [0,1]
       \return the mean intensity of top p*100 percent element in the input data vector
    */
    float meanOfTopByPercentage(vector<float> va, float p);
    
    /**
       \return the mininum value of the input vector
    */
    template <class T>
    T minimum(const vector<T> &a) {
      if (a.empty()) return T();      
      return *min_element(a.begin(), a.end());
    }
    
    /**
       \return the maximum value of the input vector
    */
    template <class T>
    T maximum(const vector<T> &a) {
      if (a.empty()) return T();
      return *max_element(a.begin(), a.end());
    }
    
    /**
       \return the mean value of the input vector
    */
    template <class T>
    float mean(const vector<T> &a) {
      if (a.empty()) return 0;
      float m(0);
      const int n(a.size());
      for(int i=0; i<n; i++) m += a[i];
      return m/(float)n;
    }
    
    /**
       \return the variance of the input vector
    */
    template <class T>
    float var(const vector<T> &a) {
      float m (0), v (0);
      if (a.empty()) return v;
      const int n(a.size());
      for(int i=0; i<n; i++) m += a[i];
      m /= (float)n;
      float d;
      for(int i=0; i<n; i++) {
	d = a[i]-m;
	v += d*d;
      }
      if (n>1) return v/(float)(n-1);
      return 0;
    }
    
    /**
       \return the median value of the input vector
    */
    template <class T>
    T median(const vector<T> &a) {
      if (a.empty()) return T();
      vector<T> b(a);
      std::sort(b.begin(), b.end());    
      return b[ b.size()/2];
    }
    
    /**
       \param a input vector
       \param m output the mean of the input vector
       \param v output the variance of the input vector
    */
    template <class T>
    void basic(const vector<T> &a, float &m, float &v) {
      m = 0; v = 0;
      if (a.empty()) return;
      const int n(a.size());
      for(int i=0; i<n; i++) m += a[i];
      m /= (float)n;
      float d;
      for(int i=0; i<n; i++) {
	d = a[i]-m;
	v += d*d;
      }
      if (n>1) v /= (float)(n-1); 
    }
    
  } // image_math

} // stdi

#endif // IMAGE_MATH_H

